'use strict'

const path = require('path')

/**
 * @const {Object} defaults - Provide some common configuration defaults
*/
const defaults = {
  React: {
    distPath: 'react/dist/react.js'
  },
  ReactDOM: {
    distPath: 'react-dom/dist/react-dom.js',
    importReact: true
  },
  ReactRouter: {
    distPath: 'react-router/umd/ReactRouter.js',
    importReact: true
  }
}

/**
 * Create a minimal base webpack configuration
 * @return {Object}
*/
const baseConfig = () => ({
  module: {
    noParse: []
  },
  resolve: {
    alias: {}
  }
})

/**
 * Returns a function that mutates a webpack configuration
 * @param {boolean|Object[]|String[]} libs - All, Path, or Array of Paths(s) to UMD builds
 * @param {string}  libs[].distPath - String of path to dist (required)
 * @param {boolean} libs[].importReact - Whether or not to expose react
 * @param {boolean} useMin - Use minified lib (default=true in `production`)
 */
function noParse (libs, opts) {

  // Shortcut syntax, use defaults
  if (typeof libs === 'boolean' && libs === true) {
    libs = [
      defaults.React,
      defaults.ReactDOM,
      defaults.ReactRouter
    ]
  }

  /**
   * Mutates json web
   * @param {Object}  config - JSON webpack configuration (required)
   */
  return function mutatorFn (config) {
    if (Array.isArray(libs) === false) {
      libs = [libs]
    }

    if (typeof opts === 'boolean' && opts === true) {
      opts = { useMin: true }
    }

    if (typeof opts !== 'undefined' && typeof opts !== 'object') {
      throw new TypeError(
        'Expected `options` to be a boolean or object, received: ' + opts
      )
    }

    const defaultOptions = {
      useMin: false
    }

    const options = Object.assign({}, defaultOptions, opts)

    if (process.env['NODE_ENV'] === 'production') {
      options.useMin = true

      console.warn(`
        Warning: 'react-noparse' is not designed for use in production.
        In a production build, external modules should use 'config.externals'
        and load the library from a CDN in index.html.
      `)
    }

    const initialState = {
      config: Object.assign({}, baseConfig(), config),
      requiresReact: []
    }

    if (typeof config.module.noParse === 'undefined') {
      initialState.config.module['noParse'] = []
    }

    if (config.resolve && typeof config.resolve.alias === 'undefined') {
      initialState.config.resolve['alias'] = {}
    }

    // Callback for `libs.reduce`
    const processNext = (data, lib) => {
      let dist

      if (typeof lib === 'object' && typeof lib['distPath'] === 'undefined') {
        throw new Error(
          'Lib config objects must contain a `distPath` key' + lib
        )
      }

      if (typeof lib === 'string') {
        dist = lib
      } else {
        dist = lib.distPath
      }

      // Extract package name for aliasing
      const name = dist.split(path.sep)[0]

      // Flag react libs for bulding loader query
      if (lib['importReact'] === true) {
        data.requiresReact.push(name)
      }

      // Append `.min` to dist filename in production
      const useMin = options['useMin']
      const fullPath = useMin ? dist.replace(/(.*)\.js$/g, '$1.min.js') : dist

      // Tell webpack not to parse or bundle the provided libraries
      const noParseTest = escapeRegExp(fullPath)
      data.config.module.noParse.push(new RegExp(`\/${name}\/`, 'g'))

      // Since webpack is skipping these paths, we need to let it know
      // where to find the modules by creating aliases that will resolve
      // to the distribution (note: must be a true UMD distribution)
      data.config.resolve.alias[`${name}$`] = `/node_modules/${fullPath}`

      return data
    }

    const results = libs.reduce(processNext, initialState)

    // Expose React to modules that require it
    // Note: If using react at all, always required for `react-dom`
    if (results.requiresReact.length > 0) {

      if(typeof results.config.module.loaders === 'undefined') {
        results.config.module['loaders'] = []
      }

      // Build regex test for importsLoader
      const names = results.requiresReact.join('|')
      const min = options.useMin ? '(\\\.min)?' : ''
      const loaderTest = new RegExp(`(${names})${min}\\\.js$`)

      const importsLoader = {
        test: loaderTest,
        loader: 'imports?React=react'
      }

      results.config.module.loaders.push(importsLoader);
    }

    return results.config
  }
}

module.exports = { baseConfig, defaults, noParse }

function escapeRegExp(str) {
  return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
}
