const assert = require('assert')
const lib = require('./index')

const baseConfig = lib.baseConfig
const defaults = lib.defaults
const noParse = lib.noParse

const fixtures = {
  lib: [
    defaults.React,
    defaults.ReactDOM,
    defaults.ReactRouter
  ],
  setupString: 'mathjs/dist/math.js',
  setupObject: {
    distPath: 'mathjs/dist/math.js'
  },
  setupInvalid: {
    dist: 'mathjs/dist/math.js'
  }
}


const tests = [
  {
    it: 'should return a webpack configuration',
    run (done) {
      const config = baseConfig()
      const actual = noParse(fixtures.lib)(config)
      const expected = {
        module: {
          noParse: [/\/react\//g, /\/react-dom\//g, /\/react-router\//g],
          loaders: [{
            test: /(react-dom|react-router)\.js$/,
            loader: 'imports?React=react'
          }]
        },
        resolve: {
          alias: {
            'react$': '/node_modules/react/dist/react.js',
            'react-dom$': '/node_modules/react-dom/dist/react-dom.js',
            'react-router$': '/node_modules/react-router/umd/ReactRouter.js'
          }
        }
      }
      assert.deepStrictEqual(actual, expected, this.it)
      done()
    }
  },

  {
    it: 'should work with shorthand syntax',
    run (done) {
      const config = baseConfig()
      const actual = noParse(true)(config)
      const expected = {
        module: {
          noParse: [/\/react\//g, /\/react-dom\//g, /\/react-router\//g],
          loaders: [{
            test: /(react-dom|react-router)\.js$/,
            loader: 'imports?React=react'
          }]
        },
        resolve: {
          alias: {
            'react$': '/node_modules/react/dist/react.js',
            'react-dom$': '/node_modules/react-dom/dist/react-dom.js',
            'react-router$': '/node_modules/react-router/umd/ReactRouter.js'
          }
        }
      }
      assert.deepStrictEqual(actual, expected, this.it)
      done()
    }
  },

  {
    it: 'should use minified libs when specified in config',
    run (done) {
      const config = baseConfig()
      const actual = noParse(fixtures.lib, { useMin: true })(config)
      const expected = {
        module: {
          noParse: [/\/react\//g, /\/react-dom\//g, /\/react-router\//g],
          loaders: [{
            test: /(react-dom|react-router)(\.min)?\.js$/,
            loader: 'imports?React=react'
          }]
        },
        resolve: {
          alias: {
            'react$': '/node_modules/react/dist/react.min.js',
            'react-dom$': '/node_modules/react-dom/dist/react-dom.min.js',
            'react-router$': '/node_modules/react-router/umd/ReactRouter.min.js'
          }
        }
      }
      assert.deepStrictEqual(actual, expected, this.it)
      done()
    }
  },

  {
    it: 'should use minified libs with shorthand syntax',
    run (done) {
      const config = baseConfig()
      const actual = noParse(fixtures.lib, true)(config)
      const expected = {
        module: {
          noParse: [/\/react\//g, /\/react-dom\//g, /\/react-router\//g],
          loaders: [{
            test: /(react-dom|react-router)(\.min)?\.js$/,
            loader: 'imports?React=react'
          }]
        },
        resolve: {
          alias: {
            'react$': '/node_modules/react/dist/react.min.js',
            'react-dom$': '/node_modules/react-dom/dist/react-dom.min.js',
            'react-router$': '/node_modules/react-router/umd/ReactRouter.min.js'
          }
        }
      }
      assert.deepStrictEqual(actual, expected, this.it)
      done()
    }
  },

  {
    it: 'should use minified libs automatically in production',
    run (done) {
      const warnings = []

      console = {}
      console.warn = msg => warnings.push(msg)

      process.env['NODE_ENV'] = 'production'

      const config = baseConfig()
      const actual = noParse(fixtures.lib)(config)
      const expected = {
        module: {
          noParse: [/\/react\//g, /\/react-dom\//g, /\/react-router\//g],
          loaders: [{
            test: /(react-dom|react-router)(\.min)?\.js$/,
            loader: 'imports?React=react'
          }]
        },
        resolve: {
          alias: {
            'react$': '/node_modules/react/dist/react.min.js',
            'react-dom$': '/node_modules/react-dom/dist/react-dom.min.js',
            'react-router$': '/node_modules/react-router/umd/ReactRouter.min.js'
          }
        }
      }
      assert.deepStrictEqual(actual, expected, this.it)
      assert.equal(warnings.length, 1)

      process.env['NODE_ENV'] = 'test'
      done()
    }
  },

  {
    it: 'should not use imports loader unless necessary',
    run (done) {
      const config = baseConfig()
      const actual = noParse(fixtures.setupString)(config)
      const expected = {
        module: { noParse: [/\/mathjs\//g] },
        resolve: {
          alias: { 'mathjs$': '/node_modules/mathjs/dist/math.js' }
        }
      }
      assert.deepStrictEqual(actual, expected, this.it)
      done()
    }
  },

  {
    it: 'should accept a single config object',
    run (done) {
      const config = baseConfig()
      const actual = noParse(fixtures.setupObject)(config)
      const expected = {
        module: { noParse: [/\/mathjs\//g] },
        resolve: {
          alias: { 'mathjs$': '/node_modules/mathjs/dist/math.js' }
        }
      }
      assert.deepStrictEqual(actual, expected, this.it)
      done()
    }
  },

  {
    it: 'should throw if config object does not have distPath',
    run (done) {
      const config = baseConfig()
      assert.throws(
        () => noParse(fixtures.setupInvalid)(config),
        (err) => ((err instanceof Error) && /Lib config objects must contain a `distPath` key/.test(err)),
        this.it
      )
      done()
    }
  },

  {
    it: 'should throw if options is not an object',
    run (done) {
      const config = baseConfig()
      assert.throws(
        () => noParse(fixtures.defaults, 'useMin')(config),
        (err) => ((err instanceof TypeError) && /Expected `options` to be a boolean or object/.test(err)),
        this.it
      )
      done()
    }
  },

  {
    it: 'should not overwrite existing configuration',
    run (done) {
      const config = baseConfig()
      config.resolve.root = 'src'
      config.module['loaders'] = []
      config.module.loaders.push({ test: /x/, loader: 'x' })

      const actual = noParse(true)(config)

      const expected = {
        module: {
          noParse: [/\/react\//g, /\/react-dom\//g, /\/react-router\//g],
          loaders: [{
            test: /x/,
            loader: 'x'
          }, {
            test: /(react-dom|react-router)\.js$/,
            loader: 'imports?React=react'
          }]
        },
        resolve: {
          root: 'src',
          alias: {
            'react$': '/node_modules/react/dist/react.js',
            'react-dom$': '/node_modules/react-dom/dist/react-dom.js',
            'react-router$': '/node_modules/react-router/umd/ReactRouter.js'
          }
        }
      }

      assert.deepStrictEqual(actual, expected, this.it)
      done()
    }
  },

  {
    it: 'should work if base has no resolvers',
    run (done) {
      const config = baseConfig()

      delete config.resolve.alias
      delete config.module.noParse

      const actual = noParse(true)(config)
      const expected = {
        module: {
          noParse: [/\/react\//g, /\/react-dom\//g, /\/react-router\//g],
          loaders: [{
            test: /(react-dom|react-router)\.js$/,
            loader: 'imports?React=react'
          }]
        },
        resolve: {
          alias: {
            'react$': '/node_modules/react/dist/react.js',
            'react-dom$': '/node_modules/react-dom/dist/react-dom.js',
            'react-router$': '/node_modules/react-router/umd/ReactRouter.js'
          }
        }
      }
      assert.deepStrictEqual(actual, expected, this.it)
      done()
    }
  }

]

const run = (log => tests => {
  log(`\n Running ${tests.length} tests...`)
  tests.map(test => test.run(result => log(' ✓ ', test.it)))
  log(`\n All ${tests.length} tests passed. Mazel! \n`)
})(console.log)

module.exports = run(tests)
